@@ -21,7 +21,7 @@
  */
 /*LICENSE_END*/
 
-
+#include <array>
 #include <CaretObject.h>
 
 #include <AString.h>
@@ -47,6 +47,12 @@ public:
     virtual ~BoundingBox();
 
 public:
+    bool isValid() const;
+    
+    bool isValid2D() const;
+    
+    bool isValidTwoAxis() const;
+    
     void resetZeros();
 
     void resetWithMaximumExtent();
@@ -64,8 +70,25 @@ public:
 
     void set(const float minMaxXYZ[6]);
 
+    void set(const float a[3],
+             const float b[3],
+             const float c[3],
+             const float d[4]);
+    
     void update(const float xyz[]);
 
+    void update(const float x,
+                const float y,
+                const float z);
+    
+    void update(const std::array<float, 3>& xyz);
+    
+    void updateExcludeNanInf(const float xyz[]);
+    
+    void updateExcludeNanInf(const float x,
+                             const float y,
+                             const float z);
+
     const float* getBounds() const;
 
     void getBounds(float bounds[6]) const;
@@ -76,6 +99,8 @@ public:
 
     float getDifferenceZ() const;
 
+    float getMaximumDifferenceOfXYZ() const;
+    
     float getMinX() const;
 
     float getMaxX() const;
@@ -88,9 +113,9 @@ public:
 
     float getMaxZ() const;
 
-    float* getMinXYZ() const;
+    std::array<float, 3> getMinXYZ() const;
 
-    float* getMaxXYZ() const;
+    std::array<float, 3> getMaxXYZ() const;
 
     void setMinX(const float value);
 
@@ -118,6 +143,10 @@ public:
     
     void limitCoordinateToBoundingBox(double xyz[3]) const;
     
+    bool intersectsXY(const BoundingBox& bb) const;
+    
+    void unionOperation(const BoundingBox& bb);
+    
     AString toString() const;
 
 private:
@@ -126,6 +155,13 @@ private:
     void copyHelper(const BoundingBox& bo);
     
 private:
+    void setPrivate(const float minX,
+                    const float maxX,
+                    const float minY,
+                    const float maxY,
+                    const float minZ,
+                    const float maxZ);
+
     float boundingBox[6];
 
 };
