@@ -21,10 +21,17 @@
 #include <cstdlib>
 #include <iostream>
 #include <sstream>
+#ifdef CARET_OS_WINDOWS
+#include <windows.h>
+#else
 #include <unistd.h>
+#endif
 
 #include <QDateTime>
 #include <QDir>
+#include <QHostInfo>
+#include <QRegularExpression>
+#include <QSysInfo>
 #include <QThread>
 #include <QUuid>
 
@@ -36,8 +43,10 @@
 #include "Windows.h"
 #endif
 
+#include "CaretAssert.h"
 #include "CaretCommandLine.h"
 #include "CaretLogger.h"
+#include "InfoItem.h"
 #include "SystemUtilities.h"
 
 using namespace caret;
@@ -437,10 +446,17 @@ SystemUtilities::relativePath(
     }
 #endif
     
-    QStringList otherPath = QDir::cleanPath(otherPathIn).split(QRegExp("[/\\\\]"), 
+#if QT_VERSION >= 0x060000
+    QStringList otherPath = QDir::cleanPath(otherPathIn).split(QRegularExpression("[/\\\\]"),
+                                                               Qt::SkipEmptyParts);
+    QStringList myPath = QDir::cleanPath(myPathIn).split(QRegularExpression("[/\\\\]"),
+                                                         Qt::SkipEmptyParts);
+#else
+    QStringList otherPath = QDir::cleanPath(otherPathIn).split(QRegularExpression("[/\\\\]"),
                                                                QString::SkipEmptyParts);
-    QStringList myPath = QDir::cleanPath(myPathIn).split(QRegExp("[/\\\\]"), 
-                                                         QString::SkipEmptyParts);    
+    QStringList myPath = QDir::cleanPath(myPathIn).split(QRegularExpression("[/\\\\]"),
+                                                         QString::SkipEmptyParts);
+#endif
     
     const unsigned int minLength = std::min(myPath.size(), otherPath.size());
     
@@ -462,8 +478,13 @@ SystemUtilities::relativePath(
     if (sameCount == 0) {
         result = otherPathIn;
     }
-    
-    //const char separator[2] = { QDir::separator(), '\0' };
+
+    /*
+     * Always use a forward slash for the separator, even on Windows,
+     * so that paths are valid on Unix (Qt does not handle backslashes
+     * properly on Unix)
+     */
+    const AString separatorCharacter("/");
     
     //
     // Is other path a subdirectory of mypath
@@ -473,7 +494,7 @@ SystemUtilities::relativePath(
         for (int j = sameCount; j < otherPath.size(); j++) {
             result.append(otherPath[j]);
             if (j < (otherPath.size() - 1)) {
-                result.append(QDir::separator());
+                result.append(separatorCharacter);
             }
         }
     }
@@ -485,12 +506,12 @@ SystemUtilities::relativePath(
     for (int j = sameCount; j < myPath.size(); j++) {
         result.append("..");
         if (j < (myPath.size() - 1)) {
-            result.append(QDir::separator());
+            result.append(separatorCharacter);
         }
     }
     for (int k = sameCount; k < otherPath.size(); k++) {
         if (result.isEmpty() == false) {
-            result.append(QDir::separator());
+            result.append(separatorCharacter);
         }
         result.append(otherPath[k]);
     }   
@@ -498,6 +519,7 @@ SystemUtilities::relativePath(
     return result;
 }
 
+#if __cplusplus < 201703L
 /**
  * Unexpected handler
  */
@@ -506,17 +528,16 @@ static void unexpectedHandler()
     std::cerr << "While running '" << caret_global_commandLine << "':" << std::endl;
     std::cerr << std::endl;
     std::cerr << "ERROR: unhandled exception." << std::endl;
-    //if (theMainWindow != NULL) {
-        const AString msg("Workbench will be terminating due to an unexpected exception.\n"
-                          "abort() will be called and a core file may be created.");
+    const AString msg("Workbench will be terminating due to an unexpected exception.\n"
+                      "abort() will be called and a core file may be created.");
     std::cerr << msg << std::endl;
-        //QMessageBox::critical(theMainWindow, "ERROR", msg);
-    //}
     
     std::cerr << SystemUtilities::getBackTrace() << std::endl;
     
     abort();
 }
+#endif
+
 
 /**
  * New handler
@@ -549,18 +570,30 @@ static void newHandler()
 }
 
 /**
- * Set handlers for unhandled exceptions or
- * out of memory errors.
+ * Set the handler for an unexpected (uncaught) exception.
  */
-void 
-SystemUtilities::setHandlersForUnexpected()
+void
+SystemUtilities::setUnexpectedHandler()
 {
+#if __cplusplus < 201703L
+    /* Removed in C++17 */
     std::set_unexpected(unexpectedHandler);
-    std::set_new_handler(newHandler);
-
+#endif
 }
 
-
+/**
+ * Set the handler for when "operator new" is unable to allocate memory.
+ * This new handler will print a message to the terminal containing a
+ * backtrace and then calls abort to end the program.
+ *
+ * NOTE: If this new handler is set, "operator new" WILL NOT
+ * throw a std::bad_alloc exception.
+ */
+void
+SystemUtilities::setNewHandler()
+{
+    std::set_new_handler(newHandler);
+}
 /**
  * Return the current directory as indicated 
  * by the system.  In most cases, use the
@@ -640,3 +673,95 @@ SystemUtilities::getWorkbenchHome()
     return workbenchHomeDirectory  = QDir::toNativeSeparators(workbenchHomeDirectory);    
 }
 
+/**
+ * @return The fully qualifed domain name for the local host.
+ *
+ *         If failure, an empty string is returned and a message
+ *         is sent to the logger.
+ * 
+ *         If the address is a "loopback" (127.0.0.1) it typically 
+ *         indicates that there is not a valid network connection
+ *         and an empty string is returned and a message is logged.
+ */
+AString
+SystemUtilities::getLocalHostName()
+{
+    /*
+     * NOTE: THIS FUNCTION DOES NOT SEEM TO WORK ON MACOS 10.14 MOJAVE.
+     * THE LOCALHOST NAME IS <XXX>.LOCAL WHERE <XXX> IS THE COMPUTER'NAME
+     * AND QHostInfo::fromName() DOES NOT MAKE IT FULLY QUALIFIED
+     */
+    QString hostName;
+    
+    QHostInfo hostInfo = QHostInfo::fromName(QHostInfo::localHostName());
+    if (hostInfo.error() == QHostInfo::NoError) {
+        for (auto address : hostInfo.addresses()) {
+            if (address.isLoopback()) {
+                CaretLogFine("Lookup of host found loopback address (localhost).  May not have valid network connection");
+                return "";
+            }
+        }
+        hostName = hostInfo.hostName();
+    }
+    else {
+        CaretLogWarning("Lookup of local host name failed: "
+                        + hostInfo.errorString());
+    }
+    
+    return hostName;
+}
+
+/**
+ * @return system information with separate names and values.
+ */
+std::vector<std::unique_ptr<InfoItem>>
+SystemUtilities::getSystemInfo()
+{
+    std::vector<std::unique_ptr<InfoItem>> dataOut;
+        
+    const AString buildApiTT("Returns the full architecture string that Qt was compiled for. "
+                             "This string is useful for identifying different, incompatible "
+                             "builds. For example, it can be used as an identifier to request "
+                             "an upgrade package from a server.");
+    dataOut.push_back(InfoItem::makeItem("Compiled on ABI: ",
+                                                 QSysInfo::buildAbi(),
+                                                 buildApiTT));
+    
+    const AString cpuTT("Returns the architecture of the CPU that the application is running "
+                        "on, in text format. Note that this function depends on what the OS "
+                        "will report and may not detect the actual CPU architecture if the OS "
+                        "hides that information or is unable to provide it. For example, a "
+                        "32-bit OS running on a 64-bit CPU is usually unable to determine "
+                        "the CPU is actually capable of running 64-bit programs.");
+    dataOut.push_back(InfoItem::makeItem("Curent CPU: ",
+                                                 QSysInfo::currentCpuArchitecture(),
+                                                 cpuTT));
+    
+    const AString kernelTT("Returns the type of the operating system kernel Qt was compiled "
+                           "for. It's also the kernel the application is running on, unless "
+                           "the host operating system is running a form of compatibility or "
+                           "virtualization layer.");
+    dataOut.push_back(InfoItem::makeItem("Kernel: ",
+                                                 QSysInfo::kernelType(),
+                                                 kernelTT));
+    
+    const AString kernVerTT("Returns the release version of the operating system kernel. "
+                            "On Windows, it returns the version of the NT kernel. On Unix "
+                            "systems, including Android and macOS, it returns the same as "
+                            "the uname -r command would return.");
+    dataOut.push_back(InfoItem::makeItem("Version: ",
+                                                 QSysInfo::kernelVersion(),
+                                                 kernVerTT));
+    
+    const AString osTT("Returns a prettier form of productType() and productVersion(), "
+                       "containing other tokens like the operating system type, codenames "
+                       "and other information. The result of this function is suitable for "
+                       "displaying to the user, but not for long-term storage, as the "
+                       "string may change with updates to Qt.");
+    dataOut.push_back(InfoItem::makeItem("O/S: ",
+                                                 QSysInfo::prettyProductName(),
+                                                 osTT));
+
+    return dataOut;
+}
+
