@@ -58,7 +58,8 @@ using namespace caret;
  * Constructor.
  */
 FociFile::FociFile()
-: CaretDataFile(DataFileTypeEnum::FOCI)
+: CaretDataFile(DataFileTypeEnum::FOCI),
+GroupAndNameHierarchyUserInterface()
 {
     initializeFociFile();
 }
@@ -88,7 +89,8 @@ FociFile::~FociFile()
  *    Object that is copied.
  */
 FociFile::FociFile(const FociFile& obj)
-: CaretDataFile(obj)
+: CaretDataFile(obj),
+GroupAndNameHierarchyUserInterface(obj)
 {
     initializeFociFile();
     copyHelperFociFile(obj);
@@ -117,7 +119,7 @@ FociFile::initializeFociFile()
 {
     m_classColorTable = new GiftiLabelTable();
     m_nameColorTable  = new GiftiLabelTable();
-    m_classNameHierarchy = new GroupAndNameHierarchyModel();
+    m_classNameHierarchy = new GroupAndNameHierarchyModel(this);
     m_metadata = new GiftiMetaData();
     m_forceUpdateOfGroupAndNameHierarchy = true;
 }
@@ -136,7 +138,7 @@ FociFile::copyHelperFociFile(const FociFile& ff)
     if (m_classNameHierarchy != NULL) {
         delete m_classNameHierarchy;
     }
-    m_classNameHierarchy = new GroupAndNameHierarchyModel();
+    m_classNameHierarchy = new GroupAndNameHierarchyModel(this);
     
     *m_metadata = *ff.m_metadata;
     
@@ -510,7 +512,7 @@ FociFile::readFile(const AString& filename)
     checkFileReadability(filename);
     
     FociFileSaxReader saxReader(this);
-    std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
+    std::unique_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
     try {
         parser->parseFile(filename, &saxReader);
     }
@@ -738,4 +740,11 @@ FociFile::addToDataFileContentInformation(DataFileContentInformation& dataFileIn
     }
 }
 
-
+/**
+ * Called when a group and name hierarchy item has attribute/status changed
+ */
+void
+FociFile::groupAndNameHierarchyItemStatusChanged()
+{
+    
+}
