@@ -54,6 +54,8 @@ using namespace caret;
 /**
  * Constructor.
  *
+ * @param mode
+ *     The conversion mode
  * @param xyz
  *     The XYZ vertices.
  * @param floatRGBA
@@ -63,10 +65,12 @@ using namespace caret;
  * @param vertexPrimitiveRestartIndices
  *     Contains indices at which the primitive should restart.
  *     The primitive will stop at the index and not connect to the next index.
- * @param lineThicknessPixels
- *     Thickness of lines in pixels.
+ * @param lineThickness
+ *     Thickness of lines in (pixels when drawingSpace is WINDOW)
  * @param colorType
  *     Type of color (solid or per-vertex)
+ * @param drawingSpace
+ *     Drawing space
  * @param lineType
  *     Type of lines drawn.
  * @param joinType
@@ -76,16 +80,18 @@ GraphicsOpenGLPolylineTriangles::GraphicsOpenGLPolylineTriangles(const std::vect
                                                                  const std::vector<float>& floatRGBA,
                                                                  const std::vector<uint8_t>& byteRGBA,
                                                                  const std::set<int32_t>& vertexPrimitiveRestartIndices,
-                                                                 const float lineThicknessPixels,
+                                                                 const float lineThickness,
                                                                  const ColorType colorType,
+                                                                 const DrawingSpace drawingSpace,
                                                                  const LineType lineType,
                                                                  const JoinType joinType)
 : m_inputXYZ(xyz),
 m_inputFloatRGBA(floatRGBA),
 m_inputByteRGBA(byteRGBA),
 m_vertexPrimitiveRestartIndices(vertexPrimitiveRestartIndices),
-m_lineThicknessPixels(lineThicknessPixels),
+m_lineThickness(lineThickness),
 m_colorType(colorType),
+m_drawingSpace(drawingSpace),
 m_lineType(lineType),
 m_joinType(joinType)
 {
@@ -114,7 +120,35 @@ GraphicsOpenGLPolylineTriangles::~GraphicsOpenGLPolylineTriangles()
  */
 GraphicsPrimitive*
 GraphicsOpenGLPolylineTriangles::convertWorkbenchLinePrimitiveTypeToOpenGL(const GraphicsPrimitive* primitive,
-                                                                     AString& errorMessageOut)
+                                                                           AString& errorMessageOut)
+{
+    return convertWorkbenchLinePrimitiveTypeToOpenGL(primitive,
+                                                     NULL,
+                                                     errorMessageOut);
+}
+
+/**
+ * Convert a Workbench Line Primitive to an OpenGL primitive.
+ * OpenGL has limits on the width of the lines that it draws.  This method will
+ * convert the lines into polygons so that any line width may be drawn.
+ *
+ * @param mode
+ *     Mode for conversion
+ * @param primitive
+ *     A graphics primitive with a primitive type that is one of the
+ *     WORKBENCH_LINE* types.
+ * @param triangleVertexIndicesToLineVertexIndicesOut
+ *     Allows conversion from a triangle vertex index to original line vertex index (ignored if NULL)
+ * @param errorMessageOut
+ *     Upon exit contains an error message if conversion failed.
+ * @return
+ *     A graphics primitive with an OPENGL_TRIANGLES type that draws the lines
+ *     using triangles.  NULL if error.
+ */
+GraphicsPrimitive*
+GraphicsOpenGLPolylineTriangles::convertWorkbenchLinePrimitiveTypeToOpenGL(const GraphicsPrimitive* primitive,
+                                                                           std::vector<int32_t>* triangleVertexIndicesToLineVertexIndicesOut,
+                                                                           AString& errorMessageOut)
 {
     CaretAssert(primitive);
     errorMessageOut.clear();
@@ -158,6 +192,7 @@ GraphicsOpenGLPolylineTriangles::convertWorkbenchLinePrimitiveTypeToOpenGL(const
             break;
     }
     
+    DrawingSpace drawingSpace = DrawingSpace::WINDOW;
     LineType lineType = LineType::LINES;
     JoinType joinType = JoinType::NONE;
     switch (primitive->m_primitiveType) {
@@ -182,6 +217,30 @@ GraphicsOpenGLPolylineTriangles::convertWorkbenchLinePrimitiveTypeToOpenGL(const
         case GraphicsPrimitive::PrimitiveType::OPENGL_TRIANGLES:
             errorMessageOut = "Input type is OPENGL_TRIANGLES but must be one of the POLYGONAL_LINE* types";
             break;
+        case GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINE_LOOP_BEVEL_JOIN:
+            drawingSpace = DrawingSpace::MODEL_XY;
+            lineType = LineType::LINE_LOOP;
+            joinType = JoinType::BEVEL;
+            break;
+        case GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINE_LOOP_MITER_JOIN:
+            drawingSpace = DrawingSpace::MODEL_XY;
+            lineType = LineType::LINE_LOOP;
+            joinType = JoinType::MITER;
+            break;
+        case GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINE_STRIP_BEVEL_JOIN:
+            drawingSpace = DrawingSpace::MODEL_XY;
+            lineType = LineType::LINE_STRIP;
+            joinType = JoinType::BEVEL;
+            break;
+        case GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINE_STRIP_MITER_JOIN:
+            drawingSpace = DrawingSpace::MODEL_XY;
+            lineType = LineType::LINE_STRIP;
+            joinType = JoinType::MITER;
+            break;
+        case GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINES:
+            drawingSpace = DrawingSpace::MODEL_XY;
+            lineType = LineType::LINES;
+            break;
         case GraphicsPrimitive::PrimitiveType::POLYGONAL_LINE_LOOP_BEVEL_JOIN:
             lineType = LineType::LINE_LOOP;
             joinType = JoinType::BEVEL;
@@ -216,11 +275,15 @@ GraphicsOpenGLPolylineTriangles::convertWorkbenchLinePrimitiveTypeToOpenGL(const
                                                    primitive->m_polygonalLinePrimitiveRestartIndices,
                                                    lineWidthPixels,
                                                    colorType,
+                                                   drawingSpace,
                                                    lineType,
                                                    joinType);
     
     GraphicsPrimitive* primitiveOut = lineConversion.convertLinesToPolygons(errorMessageOut);
-    
+
+    if (triangleVertexIndicesToLineVertexIndicesOut != NULL) {
+        *triangleVertexIndicesToLineVertexIndicesOut = lineConversion.m_triangleVertexIndicesToLineVertexIndices;
+    }
     return primitiveOut;
 }
 
@@ -328,6 +391,7 @@ GraphicsOpenGLPolylineTriangles::convertLinesToPolygons(AString& errorMessageOut
     }
     else {
         errorMessageOut = "Primitive invalid, all points may be coincident";
+        errorMessageOut.clear(); // Ignore this error message
     }
     
     return primitiveOut;
@@ -570,9 +634,18 @@ void
 GraphicsOpenGLPolylineTriangles::convertFromModelToWindowCoordinate(const float modelXYZ[3],
                                                               float windowXYZOut[3]) const
 {
-    CaretAssert(m_transformEvent);
-    m_transformEvent->transformPoint(modelXYZ,
-                                     windowXYZOut);
+    switch (m_drawingSpace) {
+        case DrawingSpace::MODEL_XY:
+            windowXYZOut[0] = modelXYZ[0];
+            windowXYZOut[1] = modelXYZ[1];
+            windowXYZOut[2] = modelXYZ[2];
+            break;
+        case DrawingSpace::WINDOW:
+            CaretAssert(m_transformEvent);
+            m_transformEvent->transformPoint(modelXYZ,
+                                             windowXYZOut);
+            break;
+    }
 }
 
 /**
@@ -607,15 +680,20 @@ GraphicsOpenGLPolylineTriangles::createTrianglesFromWindowVertices(const int32_t
         0.0f
     };
     const float lineLength = MathFunctions::normalizeVector(startToEndVector);
-    if (lineLength < 0.001f) {
-        CaretLogWarning("This should not happen: Failure to filter coincident points with indices="
-                       + AString::number(windowVertexOneIndex)
-                       + ", "
-                       + AString::number(windowVertexTwoIndex)
-                       + " Coordinates: "
-                       + AString::fromNumbers(&m_vertexWindowXYZ[iOne3], 3, ",")
-                       + "    "
-                       + AString::fromNumbers(&m_vertexWindowXYZ[iTwo3], 3, ","));
+    if (lineLength < 0.0000000001f) {
+        const AString msg("Failure to filter coincident points with indices="
+                          + AString::number(windowVertexOneIndex)
+                          + ", "
+                          + AString::number(windowVertexTwoIndex)
+                          + " Coordinates: "
+                          + AString::fromNumbers(&m_vertexWindowXYZ[iOne3], 3, ",")
+                          + "    "
+                          + AString::fromNumbers(&m_vertexWindowXYZ[iTwo3], 3, ","));
+#ifdef NDEBUG
+        CaretLogFine(msg);
+#else
+        CaretLogSevere(msg);
+#endif
     }
     
     /*
@@ -630,7 +708,7 @@ GraphicsOpenGLPolylineTriangles::createTrianglesFromWindowVertices(const int32_t
     /*
      * "Width" of rectangle
      */
-    const float halfWidth = m_lineThicknessPixels / 2.0f;
+    const float halfWidth = m_lineThickness / 2.0f;
     const float halfWidthX = perpendicularVector[0] * halfWidth;
     const float halfWidthY = perpendicularVector[1] * halfWidth;
     
@@ -1041,7 +1119,7 @@ GraphicsOpenGLPolylineTriangles::performMiterJoin(const PolylineInfo& polyOne,
     /*
      * Miter limit is a multiple of the line thickness
      */
-    const float miterLimit = m_lineThicknessPixels;
+    const float miterLimit = m_lineThickness;
     const float miterLimitSquared = miterLimit * miterLimit;
     
     /*
@@ -1214,7 +1292,7 @@ GraphicsOpenGLPolylineTriangles::getAverageColorRGBA(const int32_t primitiveVert
 void
 GraphicsOpenGLPolylineTriangles::addJoinTriangles()
 {
-    for (const auto tj : m_triangleJoins) {
+    for (const auto& tj : m_triangleJoins) {
         CaretAssertVectorIndex(m_vertexWindowXYZ, tj.m_windowVertexIndex * 3 + 2);
         const float* v1 = &m_vertexWindowXYZ[tj.m_windowVertexIndex*3];
         
@@ -1335,39 +1413,103 @@ GraphicsOpenGLPolylineTriangles::convertLineSegmentsToTriangles()
     }
     
     const bool restartEnabledFlag = true;
-    
+
+    GraphicsPrimitive* outputPrimitive(NULL);
+    switch (m_colorType) {
+        case ColorType::BYTE_RGBA_PER_VERTEX:
+        case ColorType::BYTE_RGBA_SOLID:
+            outputPrimitive = m_primitiveByteColor.get();
+            break;
+        case ColorType::FLOAT_RGBA_PER_VERTEX:
+        case ColorType::FLOAT_RGBA_SOLID:
+            outputPrimitive = m_primitiveFloatColor.get();
+            break;
+    }
+    CaretAssert(outputPrimitive);
+        
     switch (m_lineType) {
         case LineType::LINES:
         {
             const int32_t numLineSegments = (numVertices / 2);
             for (int32_t i = 0; i < numLineSegments; i++) {
+                const int32_t numOutputVerticesBefore(outputPrimitive->getNumberOfVertices() / 3);
+                
                 const int32_t i2 = i * 2;
                 createTrianglesFromWindowVertices(i2, i2 + 1);
+                
+                updateOutputVertexIndicesToInputVertexIndices(outputPrimitive,
+                                                              numOutputVerticesBefore,
+                                                              i2);
             }
         }
             break;
         case LineType::LINE_LOOP:
+        {
             for (int32_t i = 0; i < numVerticesMinusOne; i++) {
+                const int32_t numOutputVerticesBefore(outputPrimitive->getNumberOfVertices() / 3);
+                
                 CaretAssertVectorIndex(restartVertexFlags, i + 1);
                 if (restartVertexFlags[i + 1]) {
                     if (restartEnabledFlag) continue;
                 }
+
                 createTrianglesFromWindowVertices(i, i + 1);
+
+                updateOutputVertexIndicesToInputVertexIndices(outputPrimitive,
+                                                              numOutputVerticesBefore,
+                                                              i);
             }
+            
+            const int32_t numOutputVerticesBefore(outputPrimitive->getNumberOfVertices() / 3);
+            
             createTrianglesFromWindowVertices(numVerticesMinusOne, 0);
+            
+            updateOutputVertexIndicesToInputVertexIndices(outputPrimitive,
+                                                          numOutputVerticesBefore,
+                                                          numVerticesMinusOne);
+        }
             break;
         case LineType::LINE_STRIP:
+            
             for (int32_t i = 0; i < numVerticesMinusOne; i++) {
+                const int32_t numOutputVerticesBefore(outputPrimitive->getNumberOfVertices() / 3);
+                
                 CaretAssertVectorIndex(restartVertexFlags, i + 1);
                 if (restartVertexFlags[i + 1]) {
                     if (restartEnabledFlag) continue;
                 }
+                
                 createTrianglesFromWindowVertices(i, i + 1);
+                
+                updateOutputVertexIndicesToInputVertexIndices(outputPrimitive,
+                                                              numOutputVerticesBefore,
+                                                              i);
             }
             break;
     }
 }
 
+/**
+ * Update conversion from output primitive vertex indices to input vertex indices
+ *
+ * @param outputPrimitive
+ *    The output primitive
+ * @param numOutputVerticesBefore
+ *    Number of vertices prior to conversion of two points to a polyline
+ * @param inputVertexIndex
+ *    Index of the input vertex
+ */
+void
+GraphicsOpenGLPolylineTriangles::updateOutputVertexIndicesToInputVertexIndices(const GraphicsPrimitive* outputPrimitive,
+                                                                               const int32_t numOutputVerticesBefore,
+                                                                               const int32_t inputVertexIndex)
+{
+    const int32_t numOutputVerticesAfter(outputPrimitive->getNumberOfVertices() / 3);
+    for (int32_t i = numOutputVerticesBefore; i < numOutputVerticesAfter; i++) {
+        m_triangleVertexIndicesToLineVertexIndices.push_back(inputVertexIndex);
+    }
+}
+
 /**
  * Get a description of this object's content.
  * @return String describing this object's content.
