@@ -28,6 +28,7 @@
 
 #include <QJsonDocument>
 
+#include "ApplicationInformation.h"
 #include "BalsaUserRoles.h"
 #include "CaretAssert.h"
 #include "CaretHttpManager.h"
@@ -41,6 +42,7 @@
 #include "ProgramParameters.h"
 #include "Scene.h"
 #include "SceneFile.h"
+#include "SceneInfo.h"
 
 using namespace caret;
 
@@ -177,6 +179,9 @@ BalsaDatabaseManager::login(const AString& databaseURL,
     errorMessageOut = ("Login has failed.\n"
                        "HTTP Code: " + AString::number(loginResponse.m_responseCode)
                        + " Content: " + responseContent);
+    if (loginResponse.m_responseCode < 0) {
+        errorMessageOut.appendWithNewLine(loginResponse.m_errorMessage);
+    }
     logout();
 
     return false;
@@ -303,9 +308,10 @@ BalsaDatabaseManager::uploadFileWithCaretHttpManager(const AString& uploadURL,
 
     
     return verifyUploadFileResponse(uploadResponse.m_headers,
-                                 responseContentOut,
-                                 uploadResponse.m_responseCode,
-                                 errorMessageOut);
+                                    responseContentOut,
+                                    uploadResponse.m_responseCode,
+                                    uploadResponse.m_errorMessage,
+                                    errorMessageOut);
 }
 
 /**
@@ -317,6 +323,8 @@ BalsaDatabaseManager::uploadFileWithCaretHttpManager(const AString& uploadURL,
  *     Content from the response.
  * @param responseHttpCode
  *     HTTP code from response.
+ * @param responseErrorMessage
+ *
  * @param errorMessageOut
  *     Contains description of error.
  * @return
@@ -324,19 +332,39 @@ BalsaDatabaseManager::uploadFileWithCaretHttpManager(const AString& uploadURL,
  */
 bool
 BalsaDatabaseManager::verifyUploadFileResponse(const std::map<AString, AString>& responseHeaders,
-                                            const AString& responseContent,
-                                            const int32_t responseHttpCode,
-                                            AString& errorMessageOut) const
+                                               const AString& responseContent,
+                                               const int32_t responseHttpCode,
+                                               const AString& responseErrorMessage,
+                                               AString& errorMessageOut) const
 {
     if (responseHttpCode != 200) {
         if (responseHttpCode == 403) {
-            errorMessageOut = ("Upload failed.  Http Code="
+            errorMessageOut = ("Upload failed.  (Http Code="
                                + AString::number(responseHttpCode)
-                               + ".  You may not have ownership/permission to edit the study.");
+                               + ").\n\n"
+                               "Either you do not have ownership/permission to edit the study or "
+                               "the study has been submitted for curation.\n\n"
+                               "Use your web browser to login to BALSA to view the study and "
+                               "check its permissions.  If the the study has been submitted for "
+                               "curation, there will be a 'return for revision' option on the study.  "
+                               "Selection of 'return for 'revision' will allow you to upload your data.");
         }
         else {
             errorMessageOut = ("Upload failed.  Http Code="
-                               + AString::number(responseHttpCode));
+                               + AString::number(responseHttpCode)
+                               + ".\n"
+                               + responseErrorMessage);
+//            if (responseHttpCode < 0) {
+//                const QString s(responseContent.isEmpty()
+//                                ? "Response content is empty !!!"
+//                                : ("Response content:\n"
+//                                   + responseContent));
+//                CaretLogSevere("Invalid http response="
+//                               + AString::number(responseHttpCode)
+//                               + "\n"
+//                               + s
+//                               + "\n");
+//            }
         }
         return false;
     }
@@ -388,11 +416,13 @@ BalsaDatabaseManager::verifyUploadFileResponse(const std::map<AString, AString>&
         }
         
         QJsonParseError jsonError;
-        QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toLatin1(),
+        QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toUtf8(),
                                                              &jsonError);
         if (jsonDocument.isNull()) {
             errorMessageOut = ("Process upload response failed.  Failed to parse JSON, error:"
                                + jsonError.errorString()
+                               + " Offset="
+                               + AString::number(jsonError.offset)
                                + "\n\n"
                                "Response Content: "
                                + responseContent);
@@ -520,12 +550,12 @@ BalsaDatabaseManager::zipSceneAndDataFiles(const SceneFile* sceneFile,
     
     bool successFlag = false;
     try {
-        OperationZipSceneFile::createZipFile(sceneFileName,
+        OperationZipSceneFile::createZipFile(NULL,
+                                             sceneFileName,
                                              extractToDirectoryName,
                                              zipFileName,
                                              basePathName,
-                                             OperationZipSceneFile::PROGRESS_GUI_EVENT,
-                                             NULL);
+                                             OperationZipSceneFile::PROGRESS_GUI_EVENT);
         successFlag = true;
     }
     catch (const CaretException& e) {
@@ -620,8 +650,21 @@ BalsaDatabaseManager::processUploadedFile(SceneFile* sceneFile,
     if (uploadResponse.m_responseCode != 200) {
         errorMessageOut = ("Process Upload failed code: "
                            + QString::number(uploadResponse.m_responseCode)
-                           + "\n");
+                           + ".  ");
+        errorMessageOut.appendWithNewLine(uploadResponse.m_errorMessage);
         
+        uploadResponse.m_body.push_back('\0');
+        AString responseContent(&uploadResponse.m_body[0]);
+        if (m_debugFlag) {
+            std::cout << errorMessageOut << std::endl;
+            std::cout << "Process Upload FAILED to BALSA reply body: " << responseContent << std::endl;
+        }
+        else {
+            CaretLogFine("Process Upload FAILED to BALSA reply error message in GUI: "
+                         + errorMessageOut
+                         + "Process Upload FAILED to BALSA reply body: "
+                         + responseContent);
+        }
         return false;
     }
 
@@ -691,7 +734,7 @@ BalsaDatabaseManager::updateSceneIdsFromProcessUploadResponse(SceneFile* sceneFi
                                                               AString& errorMessageOut)
 {
     QJsonParseError jsonError;
-    QJsonDocument jsonDocument = QJsonDocument::fromJson(jsonContent.toLatin1(),
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(jsonContent.toUtf8(),
                                                          &jsonError);
     if (jsonDocument.isNull()) {
         errorMessageOut = ("Failed to parse JSON response from procssess upload, error:"
@@ -826,6 +869,211 @@ BalsaDatabaseManager::updateSceneIDs(SceneFile* sceneFile,
     return true;
 }
 
+/**
+ * Check scene IDs
+ * @param sceneFile
+ *    The scene file
+ * @param errorMessageOut
+ *    Output error message
+ * @return
+ *    True if successful, else false.
+ */
+bool
+BalsaDatabaseManager::checkSceneIDs(SceneFile* sceneFile,
+                                    AString& errorMessageOut)
+{
+    CaretAssert(sceneFile);
+    errorMessageOut.clear();
+
+    const AString studyID(sceneFile->getBalsaStudyID());
+    if (studyID.isEmpty()) {
+        errorMessageOut = "Study ID is invalid (empty)";
+        return false;
+    }
+    
+    AString sceneIdList;
+    const int32_t numScenes(sceneFile->getNumberOfScenes());
+    for (int32_t i = 0; i < numScenes; i++) {
+        const Scene* scene(sceneFile->getSceneAtIndex(i));
+        CaretAssert(scene);
+        const AString sceneID(scene->getBalsaSceneID());
+        if ( ! sceneID.isEmpty()) {
+            if ( ! sceneIdList.isEmpty()) {
+                sceneIdList.append(",");
+            }
+            sceneIdList.append(sceneID);
+        }
+    }
+    if (sceneIdList.isEmpty()) {
+        /*
+         * There are no scene IDs and that is OK
+         */
+        return true;
+    }
+    
+    const AString checkURL(m_databaseURL
+                           + "/study/checkScenes/"
+                           + studyID
+                           + "?sceneList="
+                           + sceneIdList);
+    if (m_debugFlag) {
+        std::cout << "Check Scenes URL: " << checkURL << std::endl;
+    }
+
+    errorMessageOut.clear();
+    
+    CaretHttpRequest caretRequest;
+    caretRequest.m_method = CaretHttpManager::POST_ARGUMENTS;
+    caretRequest.m_url    = checkURL;
+    caretRequest.m_headers.insert(std::make_pair("Content-Type",
+                                                 "application/x-www-form-urlencoded"));
+    caretRequest.m_headers.insert(std::make_pair("Cookie",
+                                                 getJSessionIdCookie()));
+    caretRequest.m_arguments.push_back(std::make_pair("type",
+                                                      "json"));
+    
+    CaretHttpResponse response;
+    CaretHttpManager::httpRequest(caretRequest, response);
+    
+    if (m_debugFlag) {
+        std::cout << "Check Scene IDs response Code: " << response.m_responseCode << std::endl;
+    }
+    
+    if (response.m_responseCode != 200) {
+        errorMessageOut = ("Check Scene IDs failed with HTTP code="
+                           + AString::number(response.m_responseCode)
+                           + ".  This error may be caused by failure to agree to data use terms.  ");
+        errorMessageOut.appendWithNewLine(response.m_errorMessage);
+        return false;
+    }
+    
+    response.m_body.push_back('\0');
+    const AString responseContent(&response.m_body[0]);
+    if (m_debugFlag) {
+        std::cout << "Check Scene IDs body:\n" << responseContent << std::endl << std::endl;
+    }
+
+    if ( ! processCheckSceneIdResponse(sceneFile,
+                                       responseContent,
+                                       errorMessageOut)) {
+        return false;
+    }
+    
+    
+    return true;
+}
+
+/**
+ * Process the content from checking scene IDs
+ * @param sceneFile
+ *    The scene file
+ * @param responseContent
+ *    The content
+ * @param errorMessageOut
+ *    Output error message
+ * @return
+ *    True if successful, else false.
+ */
+bool
+BalsaDatabaseManager::processCheckSceneIdResponse(SceneFile* sceneFile,
+                                                  const AString& responseContent,
+                                                  AString& errorMessageOut)
+{
+    QJsonParseError jsonError;
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toUtf8(),
+                                                         &jsonError);
+    if (jsonDocument.isNull()) {
+        errorMessageOut = ("Checking Scene IDs failed.  Failed to parse JSON, error:"
+                           + jsonError.errorString()
+                           + " Offset="
+                           + AString::number(jsonError.offset)
+                           + "\n"
+                           + responseContent);
+        return false;
+    }
+    
+    QByteArray json = jsonDocument.toJson();
+    if ( ! jsonDocument.isObject()) {
+        errorMessageOut  = ("Checking Scene IDs failed.  JSON content is not an object."
+                           "  \n"
+                            + responseContent);
+        return false;
+    }
+
+    bool validFlag(false);
+    bool saveSceneFileFlag(false);
+    
+    const QJsonObject object(jsonDocument.object());
+    const QStringList keys(object.keys());
+
+    AString updatedSceneNames;
+    
+    const QString badSceneIdsName("badSceneIds");
+    const QString newSceneIdsName("newSceneIds");
+    if (object.contains(badSceneIdsName) && object.value(badSceneIdsName).isArray()) {
+        if (object.contains(newSceneIdsName) && object.value(newSceneIdsName).isArray()) {
+            const QJsonArray badSceneIdsArray(object.value(badSceneIdsName).toArray());
+            const QJsonArray newSceneIdsArray(object.value(newSceneIdsName).toArray());
+            const int32_t numIDs(badSceneIdsArray.count());
+            
+            if (numIDs == newSceneIdsArray.count()) {
+                /*
+                 * If both arrays contain zero elements, all scene
+                 * IDs are valid
+                 */
+                if (numIDs == 0) {
+                    return true;
+                }
+
+                validFlag = true;
+                for (int32_t i = 0; i < numIDs; i++) {
+                    if (badSceneIdsArray.at(i).isString()
+                        && (newSceneIdsArray.at(i).isString())) {
+                        const AString badSceneID(badSceneIdsArray.at(i).toString());
+                        Scene* scene(sceneFile->getSceneWithSceneID(badSceneID));
+                        if (scene != NULL) {
+                            const AString newSceneID(newSceneIdsArray.at(i).toString());
+                            scene->getSceneInfo()->setBalsaSceneID(newSceneID);
+                            saveSceneFileFlag = true;
+                            
+                            updatedSceneNames.appendWithNewLine("   " + scene->getName());
+                        }
+                        else {
+                            errorMessageOut = ("Program Error: Scene not found with bad Scene ID: "
+                                               + badSceneID);
+                            return false;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    if (validFlag) {
+        if (saveSceneFileFlag) {
+            try {
+                sceneFile->writeFile(sceneFile->getFileName());
+                
+                if ( ! updatedSceneNames.isEmpty()) {
+                    m_infoMessages.appendWithNewLine("These scenes had their Scene IDs replaced (IDs used by other scenes in BALSA):");
+                    m_infoMessages.appendWithNewLine(updatedSceneNames);
+                }
+            }
+            catch (const DataFileException& dfe) {
+                errorMessageOut = ("Error writing scene file after replacing bad Scene IDs: "
+                                   + dfe.whatString());
+                return false;
+            }
+        }
+    }
+    
+    if ( ! validFlag) {
+        errorMessageOut = ("Response from BALSA for replacing Bad Scene IDs invalid: "
+                           + responseContent);
+    }
+    
+    return validFlag;
+}
 
 /**
  * Request the given number of scene IDs
@@ -878,7 +1126,8 @@ BalsaDatabaseManager::getSceneIDs(const int32_t numberOfSceneIDs,
     if (response.m_responseCode != 200) {
         errorMessageOut = ("Requesting Scene IDs failed with HTTP code="
                            + AString::number(response.m_responseCode)
-                           + ".  This error may be caused by failure to agree to data use terms.");
+                           + ".  This error may be caused by failure to agree to data use terms.  ");
+        errorMessageOut.appendWithNewLine(response.m_errorMessage);
         return false;
     }
     
@@ -900,15 +1149,21 @@ BalsaDatabaseManager::getSceneIDs(const int32_t numberOfSceneIDs,
          * Response from BALSA has Scene IDs separated by '<br>'
          * Example: vp5B<br>Z02A<BR>
          */
+#if QT_VERSION >= 0x060000
+        QStringList sl = responseContent.split("<br>",
+                                               Qt::SkipEmptyParts,
+                                               Qt::CaseInsensitive);
+#else
         QStringList sl = responseContent.split("<br>",
                                                QString::SkipEmptyParts,
                                                Qt::CaseInsensitive);
+#endif
         const int32_t receivedCount = sl.count();
         if (receivedCount != numberOfSceneIDs) {
             errorMessageOut = ("Requested "
                                + AString::number(numberOfSceneIDs)
                                + " but received "
-                               + receivedCount
+                               + AString::number(receivedCount)
                                + " IDs");
             return false;
         }
@@ -977,7 +1232,8 @@ BalsaDatabaseManager::requestStudyID(const AString& databaseURL,
     if (studyResponse.m_responseCode != 200) {
         errorMessageOut = ("Requesting study ID failed with HTTP code="
                            + AString::number(studyResponse.m_responseCode)
-                           + ".  This error may be caused by failure to agree to data use terms.");
+                           + ".  This error may be caused by failure to agree to data use terms.  ");
+        errorMessageOut.appendWithNewLine(studyResponse.m_errorMessage);
         return false;
     }
     
@@ -1001,11 +1257,13 @@ BalsaDatabaseManager::requestStudyID(const AString& databaseURL,
     }
     
     QJsonParseError jsonError;
-    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toLatin1(),
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toUtf8(),
                                                          &jsonError);
     if (jsonDocument.isNull()) {
         errorMessageOut = ("Requesting study ID failed.  Failed to parse JSON, error:"
-                           + jsonError.errorString());
+                           + jsonError.errorString()
+                           + " Offset="
+                           + AString::number(jsonError.offset));
         return false;
     }
     
@@ -1104,12 +1362,14 @@ BalsaDatabaseManager::getUserRoles(BalsaUserRoles& userRolesOut,
     CaretHttpManager::httpRequest(caretRequest, studyResponse);
     
     if (m_debugFlag) {
-        std::cout << "Request rolese response Code: " << studyResponse.m_responseCode << std::endl;
+        std::cout << "Request roles response Code: " << studyResponse.m_responseCode << std::endl;
     }
     
     if (studyResponse.m_responseCode != 200) {
         errorMessageOut = ("Requesting roles failed with HTTP code="
-                           + AString::number(studyResponse.m_responseCode));
+                           + AString::number(studyResponse.m_responseCode)
+                           + ".  ");
+        errorMessageOut.appendWithNewLine(studyResponse.m_errorMessage);
         return false;
     }
     
@@ -1133,11 +1393,13 @@ BalsaDatabaseManager::getUserRoles(BalsaUserRoles& userRolesOut,
     }
     
     QJsonParseError jsonError;
-    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toLatin1(),
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toUtf8(),
                                                          &jsonError);
     if (jsonDocument.isNull()) {
         errorMessageOut = ("Requesting roles failed.  Failed to parse JSON, error:"
-                           + jsonError.errorString());
+                           + jsonError.errorString()
+                           + " Offset="
+                           + AString::number(jsonError.offset));
         return false;
     }
     
@@ -1212,7 +1474,9 @@ BalsaDatabaseManager::getStudyExtractDirectoryPrefix(const AString& studyID,
     
     if (studyResponse.m_responseCode != 200) {
         errorMessageOut = ("Requesting extract directory failed with HTTP code="
-                           + AString::number(studyResponse.m_responseCode));
+                           + AString::number(studyResponse.m_responseCode)
+                           + ".  ");
+        errorMessageOut.appendWithNewLine(studyResponse.m_errorMessage);
         return false;
     }
     
@@ -1283,6 +1547,15 @@ BalsaDatabaseManager::getAllStudyInformation(std::vector<BalsaStudyInformation>&
     CaretHttpResponse idResponse;
     CaretHttpManager::httpRequest(caretRequest, idResponse);
     
+    idResponse.m_body.push_back('\0');
+    AString responseContent(&idResponse.m_body[0]);
+    CaretLogFine("Get All Study Information Response from "
+                 + studyIdURL
+                 + ", Response Code="
+                 + AString::number(idResponse.m_responseCode)
+                 + "\nContent:\n"
+                 + responseContent);
+    
     if (m_debugFlag) {
         std::cout << "Request all studies response Code: " << idResponse.m_responseCode << std::endl;
     }
@@ -1290,7 +1563,8 @@ BalsaDatabaseManager::getAllStudyInformation(std::vector<BalsaStudyInformation>&
     if (idResponse.m_responseCode != 200) {
         errorMessageOut = ("Requesting all study information failed with HTTP code="
                            + AString::number(idResponse.m_responseCode)
-                           + ".  This error may be caused by failure to agree to data use terms.");
+                           + ".  This error may be caused by failure to agree to data use terms.  ");
+        errorMessageOut.appendWithNewLine(idResponse.m_errorMessage);
         return false;
     }
     
@@ -1306,19 +1580,19 @@ BalsaDatabaseManager::getAllStudyInformation(std::vector<BalsaStudyInformation>&
         return false;
     }
     
-    idResponse.m_body.push_back('\0');
-    AString responseContent(&idResponse.m_body[0]);
     
     if (m_debugFlag) {
         std::cout << "Request all studies reply body:\n" << responseContent << std::endl << std::endl;
     }
     
     QJsonParseError jsonError;
-    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toLatin1(),
+    QJsonDocument jsonDocument = QJsonDocument::fromJson(responseContent.toUtf8(),
                                                          &jsonError);
     if (jsonDocument.isNull()) {
         errorMessageOut = ("Requesting all study information failed.  Failed to parse JSON, error:"
-                           + jsonError.errorString());
+                           + jsonError.errorString()
+                           + " Offset="
+                           + AString::number(jsonError.offset));
         return false;
     }
     QByteArray json = jsonDocument.toJson();
@@ -1380,7 +1654,7 @@ BalsaDatabaseManager::isStudyIDValid(const AString& studyID)
     
     if (getAllStudyInformation(studyInformation,
                                errorMessage)) {
-        for (const auto info : studyInformation) {
+        for (const auto& info : studyInformation) {
             if (info.getStudyID() == studyID) {
                 return true;
             }
@@ -1458,6 +1732,7 @@ BalsaDatabaseManager::uploadZippedSceneFile(SceneFile* sceneFile,
                                             AString& errorMessageOut)
 {
     errorMessageOut.clear();
+    m_infoMessages.clear();
     
     /*
      * Check for errors
@@ -1490,9 +1765,12 @@ BalsaDatabaseManager::uploadZippedSceneFile(SceneFile* sceneFile,
         return false;
     }
     
+    CaretLogFine("Uploading to BALSA with Version " + ApplicationInformation().getVersion());
+
     enum ProgressEnum {
         PROGRESS_NONE,
         PROGRESS_LOGIN,
+        PROGRESS_CHECK_SCENE_IDS,
         PROGRESS_ZIPPING,
         PROGRESS_UPLOAD,
         PROGRESS_PROCESS_UPLOAD,
@@ -1505,6 +1783,14 @@ BalsaDatabaseManager::uploadZippedSceneFile(SceneFile* sceneFile,
                                        "Logging in...");
     EventManager::get()->sendEvent(progressUpdate.getPointer());
     
+    
+    progressUpdate.setProgress(PROGRESS_CHECK_SCENE_IDS, "Checking Scene IDs");
+    EventManager::get()->sendEvent(progressUpdate.getPointer());
+    if ( ! checkSceneIDs(sceneFile,
+                         errorMessageOut)) {
+        return false;
+    }
+    
     progressUpdate.setProgress(PROGRESS_ZIPPING, "Zipping Scene and Data Files");
     EventManager::get()->sendEvent(progressUpdate.getPointer());
     
@@ -1707,3 +1993,11 @@ BalsaDatabaseManager::SceneFileIdentifiers::getErrorMessage() const
     return m_errorMessage;
 }
 
+/**
+ * @return Information messages from uploading
+ */
+AString
+BalsaDatabaseManager::getInfoMessages() const
+{
+    return m_infoMessages;
+}
